package MT::Plugin::Quickedit;
use strict;

use MT;
use MT::Entry;
use MT::Page;
use MT::FileInfo;
use MT::Util qw( encode_html offset_time_list );
our $VERSION = "0.6";

@MT::Plugin::Quickedit::ISA = qw( MT::Plugin );

my $plugin = new MT::Plugin::Quickedit( {
    name => 'Quickedit',
    version => $VERSION,
    description => '<MT_TRANS phrase=\'_PLUGIN_DESCRIPTION\'>',
    author_name => 'Junnama Noda',
    author_link => 'http://junnama.alfasado.net/online/',
    settings => new MT::PluginSettings ( [
        ['editor_style_css', { Default => 'tiny_mce/css/content.css' }],
        ['title_style', { Default => 'width:98%;font-size:100%;padding:2px;font-weight:bold;border:1px solid #ddd' }],
        ['text_style', { Default => 'width:99%;height:240px;font-size:105%' }],
        ['text_more_style', { Default => 'width:99%;height:200px;font-size:105%' }],
        ['add_actionbar', { Default => '<p style="margin-top:7px;">Tags : <input style="width:90%;font-size:100%;border:1px solid #ddd" type="text" name="tags" value="<mt:entrytags glue=","><mt:tagname></mt:entrytags>" /></p>' }],
        ['field_auto', { Default => 1 }],
        ['use_wysiwyg', { Default => 1 }],
    ] ),
    blog_config_template => 'quickedit_cfg.tmpl',
    l10n_class => 'Quickedit::L10N',
} );

MT->add_plugin( $plugin );

sub init_registry {
    my $plugin = shift;
    $plugin->registry( {
        applications => {
            cms => {
                methods => {
                    quickedit =>
                        'MT::Plugin::Quickedit::_quickedit',
                    quicksave =>
                        'MT::Plugin::Quickedit::_quicksave',
                },
            },
        },
        callbacks => {
            'MT::App::CMS::template_source.quickedit_cfg'
                => \&_quickedit_cfg,
        },
        tags => {
            block => {
                'IfQuickEdit?' => \&_hdlr_if_quickedit,
            },
            function => {
                'QuickEditFormHeader' => \&_hdlr_quickedit_header,
                'QuickEditFormFooter' => \&_hdlr_quickedit_footer,
            },
        },
   } );
}

sub _hdlr_if_quickedit {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    if ( $app->mode eq 'quickedit' ) {
        return 1;
    } else {
        return 0;
    }
    return 0;
}

sub _hdlr_quickedit_header {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    if ( $app->mode eq 'quickedit' ) {
        my $e = $ctx->stash( 'entry' ) 
            or return $ctx->_no_entry_error( 'QuickEditFormHeader' );
        my $form = "<form method=\"post\" action=\"" . $app->base . $app->uri . "\">";
        $form .= '<input type="hidden" name="id" value="' . $e->id . '" />';
        $form .= '<input type="hidden" name="class" value="' . $e->class . '" />';
        $form .= '<input type="hidden" name="__mode" value="quicksave" />';
        $form .= '<input type="hidden" name="blog_id" value="' . $e->blog_id . '" />';
        return $form;
    }
}

sub _hdlr_quickedit_footer {
    my ( $ctx, $args, $cond ) = @_;
    my $app = MT->instance();
    if ( $app->mode eq 'quickedit' ) {
        return '<p style="text-align:center;margin-top:7px"><input type="submit" value="' . $plugin->translate( 'Save' ) . '" /></p></form>';
    }
}

sub _quickedit_cfg {
    my ( $cb, $app, $tmpl ) = @_;
    my $blog_id = $app->param( 'blog_id' );
    my $base = $app->base;
    $$tmpl =~ s/<mt:blog_id>/$blog_id/g;
    $$tmpl =~ s/<mt:var\sname="base">/$base/g;
}

sub _quicksave {
    my $app = shift;
    # my $permalink = $app->param( 'permalink' );
    my $entry_id = $app->param( 'id' );
    my $title = $app->param( 'title' );
    my $text = $app->param( 'text' );
    my $text_more = $app->param( 'text_more' );
    my $class = $app->param( 'class' );
    my $blog_id = $app->param( 'blog_id' );
    my $e; my $original;
    if ( $entry_id ) {
        $e = MT->model( $class )->load( $entry_id );
        $original = $e->clone() if defined $e;
    } else {
        $e = MT->model( $class )->new;
        $e->blog_id( $blog_id );
        $e->author_id( $app->user->id );
        $e->status( MT::Entry::RELEASE() );
        my @tl = &offset_time_list( time, $app->blog );
        my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
        $e->authored_on( $ts );
    }
    unless ( defined $e ) {
        return $app->trans_error( 'Invalid request.' );
    }
    unless ( _can_post( $app->user, $e->blog_id, $e ) ) {
        return $app->trans_error( 'Permission denied.' );
    }
    $e->title( $title );
    $e->text( $text );
    my $text_more = $app->param( 'text_more' );
    if ( defined $text_more ) {
        $e->text_more( $text_more );
    }
    my $keywords = $app->param( 'keywords' );
    if ( defined $keywords ) {
        $e->keywords( $keywords );
    }
    my $excerpt = $app->param( 'excerpt' );
    if ( defined $excerpt ) {
        $e->keywords( $excerpt );
    }
    my $tags = $app->param( 'tags' );
    if ( defined $tags ) {
        my $blog = $app->blog;
        my $fields = $blog->smart_replace_fields;
        if ( $fields =~ m/tags/ig ) {
            $tags = MT::App::CMS::_convert_word_chars( $app, $tags );
        }
        require MT::Tag;
        my $tag_delim = chr( $app->user->entry_prefs->{ tag_delim } );
        my @tags = MT::Tag->split( $tag_delim, $tags );
        if ( @tags ) {
            $e->set_tags( @tags );
        } else {
            $e->remove_tags();
        }
    }
    $app->run_callbacks( 'cms_pre_save.' . $class, $app, $e, $original );
    $e->save or die $e->errstr;
    $app->run_callbacks( 'cms_post_save.' . $class, $app, $e, $original );
    MT::Util::start_background_task(
        sub { $app->rebuild_entry( Entry => $e,
                                   BuildDependencies => 1 );
            }
    );
    $app->redirect( $e->permalink );
}

sub _quickedit {
    my $app = shift;
    my $permalink = $app->param( 'permalink' );
    my $url = $permalink;
    $url =~ s!^https*://.*?(/.*$)!$1!;
    my @classes = ['Individual', 'Page']; 
    my $finfos = MT::FileInfo->load_iter( { url => $url,
                                            archive_type => \@classes, });
    my $e;
    while ( my $fi = $finfos->() ) {
        if ( $fi->entry_id ) {
            $e = MT::Entry->load( $fi->entry_id );
            if ( defined $e ) {
                if ( $permalink eq $e->permalink ) {
                    last;
                }
            }
        }
    }
    unless ( defined $e ) {
        if ( $app->blog ) {
            my @tl = &offset_time_list( time, $app->blog );
            my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
            my $class = 'entry';
            if ( $app->param( 'class' ) eq 'page' ) {
                $class = 'page';
            }
            $e = MT->model( $class )->new;
            $e->blog_id( $app->blog->id );
            $e->author_id( $app->user->id );
            $e->status( MT::Entry::RELEASE() );
            $e->authored_on( $ts );
        }
    }
    if ( defined $e ) {
        unless ( _can_post( $app->user, $e->blog_id, $e ) ) {
            return $app->trans_error( 'Permission denied.' );
        }
        my $class = $e->class;
        my $at;
        if ( $class eq 'entry' ) {
             $at = 'Individual';
        } elsif ( $class eq 'page' ) {
             $at = 'Page';
             $e = MT::Page->load( $e->id ) if $e->id;
        }
        my $tmplate = MT::Template->load ( { blog_id => $e->blog_id } , 
                                           { join => [ 'MT::TemplateMap', 'template_id',
                                           { blog_id => $e->blog_id,
                                             archive_type => $at,
                                             is_preferred => 1 },
                                             { unique => 1 }
                                          ], } );
        $app->trans_error( 'Can\'t load template.' ) unless defined $tmplate;
        my $lang = $e->blog->language;
        my $tiny_mce = $app->static_path . 'tiny_mce/tiny_mce.js';
        my $get_from = 'blog:'. $e->blog->id;
        my $editor_css = $app->static_path . $plugin->get_config_value( 'editor_style_css', $get_from );
        my $title_style = $plugin->get_config_value( 'title_style', $get_from );
        my $text_style = $plugin->get_config_value( 'text_style', $get_from );
        my $text_more_style = $plugin->get_config_value( 'text_more_style', $get_from );
        my $add_actionbar = $plugin->get_config_value( 'add_actionbar', $get_from );
        my $field_auto = $plugin->get_config_value( 'field_auto', $get_from );
        my $use_wysiwyg = $plugin->get_config_value( 'use_wysiwyg', $get_from );
        my $static_path = $app->static_path;
        my $blog_id = $e->blog_id;
        my $src = $tmplate->text;
        my $insertimage = $plugin->translate( 'Image' );
        my $insertfile = $plugin->translate( 'File' );
        if ( $use_wysiwyg ) {
            my $js = <<JS;
<!-- MT_JS -->
<script type="text/javascript" src="${static_path}js/mt_core_compact.js"></script>
<script type="text/javascript" src="${static_path}mt.js"></script>
<!-- /MT_JS -->
<!-- TinyMCE -->
<script type="text/javascript" src="$tiny_mce"></script>
<script type="text/javascript">
    tinyMCE.init({
        // General options
        mode : 'textareas',
        theme : 'advanced',
        language : '$lang',
        convert_urls : false,
        plugins : 'safari,pagebreak,style,layer,table,save,advhr,advimage,advlink,emotions,iespell,inlinepopups,insertdatetime,preview,media,searchreplace,print,contextmenu,paste,directionality,fullscreen,noneditable,visualchars,nonbreaking,xhtmlxtras,template',
        // Theme options
        theme_advanced_buttons1 : 'bold,italic,underline,del,ins,|,justifyleft,justifycenter,justifyright,styleselect,formatselect,styleprops,|,attribs,cleanup,code,|,fullscreen',
        theme_advanced_buttons2 : 'cut,copy,paste,pastetext,pasteword,|,search,replace,|,bullist,numlist,|,outdent,indent,blockquote,|,cite,abbr,acronym,|,link,unlink,anchor,image',
        theme_advanced_buttons3 : 'tablecontrols,|,hr,removeformat,visualaid,|,charmap,|,media,|,mt-image,|,mt-file',
        theme_advanced_buttons4 : '',
        
        theme_advanced_toolbar_location : 'bottom',
        theme_advanced_toolbar_align : 'left',
        theme_advanced_statusbar_location : 'bottom',
        theme_advanced_resizing : true,
        theme_advanced_resize_horizontal : false,
        theme_advanced_resizing_min_height : 100,

        // Example content CSS (should be your site CSS)
        content_css : '$editor_css',
/*
        // Drop lists for link/image/media/template dialogs
        template_external_list_url : 'lists/template_list.js',
        external_link_list_url : 'lists/link_list.js',
        external_image_list_url : 'lists/image_list.js',
        media_external_list_url : 'lists/media_list.js',

        // Replace values for the template plugin
        template_replace_values : {
            username : 'Some User',
            staffid : '991234'
        },
*/
        setup : function(ed) {
            ed.addButton('mt-image', {
                title : '$insertimage',
                image : '${static_path}tiny_mce/icon/photo.png',
                onclick : function() {
                    ed.bookmark = ed.selection.getBookmark();
                    app.openDialog('__mode=list_assets&amp;_type=asset&amp;edit_field='+ ed.id +'&amp;blog_id=${blog_id}&amp;dialog_view=1&amp;filter=class&amp;filter_val=image');
                }
            });
            ed.addButton('mt-file', {
                title : '$insertfile',
                image : '${static_path}tiny_mce/icon/page_white_text.png',
                onclick : function() {
                    ed.bookmark = ed.selection.getBookmark();
                    app.openDialog('__mode=list_assets&amp;_type=asset&amp;edit_field='+ ed.id +'&amp;blog_id=${blog_id}&amp;dialog_view=1');
                }
            });
        },
        extended_valid_elements : 'form[action|accept|accept-charset|enctype|method|class|style|mt::asset-id]',
        cleanup : true,
        dialog_type : 'modal'
    });
</script>
<!-- /TinyMCE -->

<script type="text/javascript">
/* <![CDATA[ */
    ScriptURI = '@{[ $app->base . $app->uri ]}';
    App.singletonConstructor =
        MT.App = new Class(MT.App, {
            insertHTML: function(html,field) {
                var editor = tinyMCE.get(field);
                editor.focus();
                editor.selection.moveToBookmark(editor.bookmark);
                editor.execCommand('mceInsertContent',false,html);
            }
        });
    App.bootstrapInline( false );
/* ]]> */
</script>

JS
            my $header = <<__HEAD__;
<style type="text/css">
#dialog-container {
    display:none;
    position:fixed;
    top:0;
    left:0;
    width:100%;
    height:100%;
    z-index:9;
}

/*\*/
* html #dialog-container {
    position: absolute;
}
/**/

.modal-mask,
.modal-mask div {
    position:absolute;
    left:0;
    top:0;
    width:100%;
    height:100%;
    background-image:url(${static_path}images/spacer.gif);
}
.modal-mask iframe {
    border:0;
}
.opaque-mask {
    position:absolute;
    left:0;
    top:0;
    width:100%;
    height:100%;
    background-image:url(${static_path}images/black.gif);
    opacity:.50;
    -moz-opacity:.50;
    filter:progid:DXImageTransform.Microsoft.Alpha(opacity=50);
}
#dialog {
    position:relative;
    margin:0 auto;
    top:40px;
    width:660px;
    height:495px;
    background-color:#fff;
}
#dialog-iframe {
    height:100%;
    width:100%;
    margin:0;
    border:0;
    padding:0;
    z-index:10;
}
.hidden {
    display:none !important;
}
</style>
__HEAD__
            $src =~ s{(</head>)}{$header$1};
            my $footer = <<__FOOT__;
<div id="dialog-container" class="autolayout-height-parent hide" style="display:none;">
    <div class="modal-mask opaque-mask"></div>
    <div id="dialog"><iframe frameborder="0" scrolling="auto" src="${static_path}html/blank.html" id="dialog-iframe" name="dialog_iframe"></iframe></div>
</div>
$js
__FOOT__
            $src =~ s{(</body>)}{$footer$1};
        }
        if ( $field_auto ) {
            my $title = encode_html( $e->title );
            my $text  = encode_html( $e->text );
            my $text_more = encode_html( $e->text_more );
            $e->text( $e->text . ' ' );
            $e->text_more( $e->text_more . ' ' );
            my $form = "<form method=\"post\" action=\"" . $app->base . $app->uri . "\">";
            $form .= '<input type="hidden" name="id" value="' . $e->id . '" />';
            $form .= '<input type="hidden" name="class" value="' . $class . '" />';
            $form .= '<input type="hidden" name="__mode" value="quicksave" />';
            $form .= '<input type="hidden" name="blog_id" value="' . $e->blog_id . '" />';
            my $title_fld = "<input type=\"text\" name=\"title\" value=\"$title\" style=\"$title_style\" />";
            my $text_fld  = "<textarea name=\"text\" style=\"$text_style\">$text</textarea>";
            my $action_bar = $add_actionbar . '<p style="text-align:center;margin-top:7px"><input type="submit" value="' . $plugin->translate( 'Save' ) . '" /></p></form>';
            my $more_fld  = "<textarea name=\"text_more\" style=\"$text_more_style\">$text_more</textarea>";
            if ( $class eq 'entry' ) {
                if ( $src =~ m/<\$*MT:*EntryMore.*?\$*>/is ) {
                    $more_fld .= $action_bar;
                } else {
                    $text_fld .= $action_bar;
                }
                $src =~ s/(<h[1-6].*?>.*?)<\$*MT:*EntryTitle.*?\$*>(.*?<\/h[1-6]>)/$form$1$title_fld$2/is;
                $src =~ s/<\$*MT:*EntryBody.*?\$*>/$text_fld/is;
                $src =~ s/<\$*MT:*EntryMore.*?\$*>/$more_fld/is;
            } elsif ( $class eq 'page' ) {
                if ( $src =~ m/<\$*MT:*PageMore.*?\$*>/is ) {
                    $more_fld .= $action_bar;
                } else {
                    $text_fld .= $action_bar;
                }
                $src =~ s/(<h[1-6].*?>.*?)<\$*MT:*PageTitle.*?\$*>(.*?<\/h[1-6]>)/$form$1$title_fld$2/is;
                $src =~ s/<\$*MT:*PageBody.*?\$*>/$text_fld/is;
                $src =~ s/<\$*MT:*PageMore.*?\$*>/$more_fld/is;
            }
        }
        my $html = _build_tmpl( $src, $at, $e, $e->blog, $e->category, $app );
        return $html;
    } else {
        $app->redirect( $app->uri );
    }
}

sub _can_post {
    my ( $user, $blog_id, $entry ) = @_;
    return 0 unless defined $user;
    my $can_post = $user->is_superuser;
    unless ( $can_post ) {
        if ( $entry->class eq 'entry' ) {
            $can_post = $user->permissions( $blog_id )->can_post;
            if ( $can_post ) {
                if ( $user->id != $entry->author_id ) {
                    $can_post = $user->permissions( $blog_id )->can_edit_all_posts;
                }
            }
        } else {
            $can_post = $user->permissions( $blog_id )->can_manage_pages;
        }
    }
    return $can_post;
}

sub _build_tmpl {
    my ( $template, $at, $entry, $blog, $category, $app ) = @_;
    my $ctx = MT::Template::Context->new;
    $ctx->{ __stash }->{ vars }->{ entry_template } = 1 if $at eq 'Individual';
    $ctx->{ __stash }->{ vars }->{ page_template }  = 1 if $at eq 'Page';
    $ctx->stash( 'entry', $entry );
    $ctx->stash( 'page', $entry );
    $ctx->stash( 'blog', $blog );
    $ctx->stash( 'category', $category );
    $ctx->stash( 'blog_id', $blog->id );
    my $build = MT::Builder->new;
    my $tokens = $build->compile( $ctx, $template )
        or return $app->error( $app->translate(
            "Parse error: [_1]", $build->errstr) );
    defined ( my $html = $build->build( $ctx, $tokens ) )
        or return $app->error( $app->translate(
            "Build error: [_1]", $build->errstr) );
    return $html;
}

1;